package com.lightinit.hsdataplatform.impl.admin;

import com.lightinit.hsdataplatform.common.GateWayUtils;
import com.lightinit.hsdataplatform.common.OrderNumberUtils;
import com.lightinit.hsdataplatform.dictionary.*;
import com.lightinit.hsdataplatform.entity.*;
import com.lightinit.hsdataplatform.mapper.*;
import com.lightinit.hsdataplatform.model.ResultState;
import com.lightinit.hsdataplatform.model.ResultStateCode;
import com.lightinit.hsdataplatform.model.admin.*;
import com.lightinit.hsdataplatform.service.admin.IAuthorizationService;
import com.lightinit.hsdataplatform.spring.DynamicDataContextHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
public class AuthorizationServiceImpl implements IAuthorizationService {

    static Logger log = LoggerFactory.getLogger(AuthorizationServiceImpl.class);
    @Autowired
    private ResourceAuthMapper resourceAuthMapper;

    @Autowired
    private PublishSettingMapper publishSettingMapper;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private OrderGoodsMapper orderGoodsMapper;

    @Autowired
    private AdminRoleMapper adminRoleMapper;

    @Autowired
    private UserAuthSettingMapper userAuthSettingMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private AdminMapper adminMapper;

    @Autowired
    private PublishInfoMapper publishInfoMapper;

    @Autowired
    private ResourceApiInterfaceMapper resourceApiInterfaceMapper;

    @Autowired
    private ResourceMapper resourceMapper;

    @Override
    public List<AuthorizationListModel> queryList(AuthorizationSearchModel inputModel) {
        DynamicDataContextHolder.setCustomerType(DynamicDataContextHolder.DEFAULT);
        return resourceAuthMapper.queryList(inputModel);
    }

    @Override
    public long queryCount(AuthorizationSearchModel inputModel) {
        DynamicDataContextHolder.setCustomerType(DynamicDataContextHolder.DEFAULT);
        return resourceAuthMapper.queryCount(inputModel);
    }

    @Override
    public AuthorizationViewModel queryOneView(long id) {
        DynamicDataContextHolder.setCustomerType(DynamicDataContextHolder.DEFAULT);
        return resourceAuthMapper.queryOneView(id);
    }

    @Override
    @Transactional
    public ResultState<Long> handle(AuthorizationHeableModel inputModel, LoginUser loginUser) {
        DynamicDataContextHolder.setCustomerType(DynamicDataContextHolder.DEFAULT);
        ResultState<Long> resultState = new ResultState<Long>();
        Date createdAt = new Date();

        //修改resource_auth表
        ResourceAuth resourceAuth = new ResourceAuth();
        resourceAuth.setAuditOpinion(inputModel.getAuditOpinion());
        resourceAuth.setAuthState(inputModel.getAuthorizationState());
        ResourceAuthExample example = new ResourceAuthExample();
        ResourceAuthExample.Criteria criteria = example.createCriteria();
        criteria.andIdEqualTo(inputModel.getId());
        int result0 = resourceAuthMapper.updateByExampleSelective(resourceAuth, example);

        int result1 = 0;
        int result2 = 0;
        if (inputModel.getAuthorizationState().equals(DicAuthorizationState.AUTHORIZE_OK)) {
            PublishSetting publishSetting = publishSettingMapper.selectByPrimaryKey(inputModel.getPublishSettingId());
            PublishInfoWithBLOBs publishInfoWithBLOBs = publishInfoMapper.selectByPrimaryKey(publishSetting.getPublishInfoId());
            Resource resource = resourceMapper.selectByPrimaryKey(publishInfoWithBLOBs.getResourceId());
            //插入order表
            Order order = new Order();
            order.setPlatformType(inputModel.getPlatfromType());
            order.setUserId(loginUser.getId());
            order.setOrderTime(createdAt);
            order.setOrderNumber(OrderNumberUtils.generate());
            if (inputModel.getPlatfromType().equals(DicOrderTypeState.OFFLINE)) {
                order.setOrderState(DicOrderState.PAID);
            } else {
                if (publishSetting.getChargeMethod().equals(DicPublishSettingChargeMethod.FREE)) {
                    order.setOrderState(DicOrderState.PAID);
                } else {
                    order.setOrderState(DicOrderState.NOTPAID);
                }
            }
            order.setOrderAmount(publishSetting.getPackagePrice());
            order.setRemark(publishSetting.getDescription());
            order.setCreatedAt(createdAt);
            result1 = orderMapper.insert(order);
            //插入order_goods表
            OrderGoods orderGoods = new OrderGoods();
            BeanUtils.copyProperties(publishSetting, orderGoods);
            orderGoods.setOrderId(order.getId());
            orderGoods.setResourceType(inputModel.getResourceType());
            orderGoods.setResourceName(inputModel.getResourceName());
            orderGoods.setPublishSettingId(inputModel.getPublishSettingId());
            orderGoods.setCreatedAt(createdAt);
            orderGoods.setResourceAuthId(inputModel.getId());
            result2 = orderGoodsMapper.insert(orderGoods);
            if (result1 > 0 && result2 > 0) {
                //=================新路由鉴权和计费==================
                if (order.getOrderState().equals(DicOrderState.PAID) && orderGoods.getResourceType().equals(DicResourceType.API)) {
                    try {
                        ResourceApiInterfaceExample example1 = new ResourceApiInterfaceExample();
                        ResourceApiInterfaceExample.Criteria criteria1 = example1.createCriteria();
                        criteria1.andApiIdEqualTo(publishInfoWithBLOBs.getResourceId());
                        List<ResourceApiInterfaceWithBLOBs> resourceApiInterfaceWithBLOBs = resourceApiInterfaceMapper.selectByExampleWithBLOBs(example1);
                        //新增网关计费
                        GateWayUtils.gatewayChargeAdd(resourceApiInterfaceWithBLOBs, loginUser.getUsername(), loginUser.getUsername() + "添加计费");
                    } catch (Exception e) {
                        e.printStackTrace();
                        log.error("同步网关失败！所在类是：AuthorizationServiceImpl,行数是：147");
                    }
                }
                if (order.getOrderState().equals(DicOrderState.PAID) && orderGoods.getResourceType().equals(DicResourceType.MICROAPP)) {
                    try {
                        //新增网关计费
                        GateWayUtils.gatewayChargeAddByMic(resource, loginUser.getUsername(), loginUser.getUsername() + "添加计费");
                    } catch (Exception e) {
                        e.printStackTrace();
                        log.error("同步网关失败！所在类是：AuthorizationServiceImpl,行数是：147");
                    }
                }

            }
        }


        if (!(result0 > 0 || (result1 > 0 && result2 > 0))) {
            resultState.setStateCode(ResultStateCode.INVALID_DATA);
            resultState.setMsg("授权失败");
        }
        return resultState;
    }

    @Override
    public int isAdminFalg(Long userId) {
        return adminRoleMapper.isAdminFalg(userId);
    }
}
